/*!
 * \file ir.c
 * \brief Interface to a infrared receiver.
 *
 * Based on buttons.c from the Elektor Webradio Project
 * expected, that the port pin is driven low when signal is there.
 *
 */

/*!
 * \example ir.c
 *
 * Connection to a IR receiver.
 *
 * \code
 * call IR_Init()
 * ...
 * NutThreadCreate("IRrecv", IR_Receiver, &xx, 512);
 *
 * \endcode
 */

#include "ir.h"

#include <cfg/os.h>
#include <cfg/clock.h>
#include <arch/arm.h>
#include <dev/irqreg.h>

#include <sys/event.h>
#include <sys/timer.h>
#include <stdio.h>

//#include "sam7x.h"				// only for showing the timing on an LED
#include "Irmp/irmp.h"				// the interpretation of the IR-data
#include "Irmp/irmpconfig.h"		// F_INTERRUPTS

/*! 
 * \def IO_TC_ID
 * \brief Button scan timer ID.
 *
 * Specifies the timer/counter that will be used to generate periodical interrupts for scanning the input.
*/
#define IO_TC_ID       TC1_ID

/*!
 * \def IO_PIO_ID
 * \brief PIO ID of the IO interface. */
#define IO_PIO_ID      PIOB_ID

/*!
 * \def IO_IR_BIT
 * \brief used PIO bit. */
#define IO_IR_BIT      29

/*!
 * \def IO_IR
 * \brief PIO bit mask for used bits. */
#define IO_IR          _BV(IO_IR_BIT)

/*!
 * \def IO_PIO_PE_REG
 * \brief PIO enable register of the button interface.
*/
#define IO_PIO_PE_REG  PIOB_PER

/*!
 * \def IO_PIO_OD_REG
 * \brief PIO output disable register of the button interface.
*/
#define IO_PIO_OD_REG  PIOB_ODR

/*!
 * \def IO_PIO_PDS_REG
 * \brief Pin data status register of the button interface.
*/
#define IO_PIO_PDS_REG PIOB_PDSR

/*!
 * \def IO_PIO_PUE_REG
 * \brief Pull-up enable register of the button interface.
 */
#define IO_PIO_PUE_REG PIOB_PUER

/*!
 * \def sig_IO_TC
 * \brief Interrupt signal of the button scan timer.
 */
#define sig_IO_TC      sig_TC1

/*!
 * \def IO_TC_CC_REG
 * \brief Channel control register of the button scan timer.
 */
#define IO_TC_CC_REG   TC1_CCR

/*!
 * \def IO_TC_CM_REG
 * \brief Channel mode register of the button scan timer.
 */
#define IO_TC_CM_REG   TC1_CMR

/*!
 * \def IO_TC_IE_REG
 * \brief Interrupt enable register of the button scan timer.
 */
#define IO_TC_IE_REG   TC1_IER

/*!
 * \def IO_TC_ID_REG
 * \brief Interrupt disable register of the button scan timer.
 */
#define IO_TC_ID_REG   TC1_IDR

/*!
 * \def IO_TC_S_REG
 * \brief Status register of the button scan timer.
 */
#define IO_TC_S_REG    TC1_SR

/*!
 * \def IO_TC_RC_REG
 * \brief Register C of the button scan timer.
 */
#define IO_TC_RC_REG   TC1_RC

/*!
 * \brief Button event queue.
 *
 * The IO scan timer interrupt handler will post an event to this
 * queue whenever a new button had been pressed or the repeat timer
 * elapses with a button still pressed.
 */
static HANDLE event_que;

static int volatile last_command;

/*!
 * \brief this is the function for irmp to get the status of the IO-Pin
 */
uint8_t input(uint8_t test)
{
	return ( (~inr(IO_PIO_PDS_REG) & IO_IR) == 0);
}

/*!
 * \brief this thread is an example for getting the received results.
 */
THREAD(IR_Receiver, arg)
{
    NutThreadSetPriority(64);
	for(;;){
		int command = IR_Read(1);
		if(command){
			printf("IR-Command:%d\r\n", command);
		}
        NutThreadYield();
	}
}

/*!
 * \brief the scan timer interrupt handler.
 *
 * This function is called by the IRQ-Handler, F_INTERRUPTS times a second
 */
static void ScanTimerInterrupt(void *arg)
{
#if 0    // setting the LED for measuring the frequency
    static unsigned char ledIsOn = 0;
	ledIsOn = !ledIsOn;
	ledIsOn ? Led4On() : Led4Off();
#endif
	//Led4On();	// for measuring the spend time for IR-interpretation

	uint8_t result = irmp_ISR();		// this here lasts 5-7us
	if(result){							// a command is received
		IRMP_DATA ir_receive;
		if(irmp_get_data (&ir_receive)){
			/* for example copy the command */
			last_command = ir_receive.command;
			NutEventPostFromIrq(&event_que);
		}
	}
	//Led4Off();
}

/*!
 */
void IR_Init(void)
{
    /* Enable scan timer clock. */
    outr(PMC_PCER, _BV(IO_TC_ID) | _BV(IO_PIO_ID));
    /* Disable the Clock Counter */
    outr(IO_TC_CC_REG, TC_CLKDIS);
    /* Disable all interrupts */
    outr(IO_TC_ID_REG, 0xFFFFFFFF);
    /* Clear the status register. */
    /*int dummy = */inr(IO_TC_S_REG);
    /* Select divider and compare trigger */
    outr(IO_TC_CM_REG, TC_CLKS_MCK32 | TC_CPCTRG);
    /* Enable the Clock counter */
    outr(IO_TC_CC_REG, TC_CLKEN);
    /* Validate the RC compare interrupt */
    outr(IO_TC_IE_REG, TC_CPCS);

    /* Register timer interrupt handler. */
    NutRegisterIrqHandler(&sig_IO_TC, ScanTimerInterrupt, 0);

    /* Set to lowest priority. */
    NutIrqSetPriority(&sig_IO_TC, 0);

    /* Enable timer interrupts */
    NutIrqEnable(&sig_IO_TC);

    /* Set compare value for specified scan frequency. */
    outr(IO_TC_RC_REG, NutGetCpuClock() / (32 * F_INTERRUPTS));

    /* Initialize GPIO lines for IR-Read. */
    outr(IO_PIO_PE_REG, IO_IR);
    outr(IO_PIO_OD_REG, IO_IR);
    outr(IO_PIO_PUE_REG, IO_IR);

    /* Software trigger starts the scan timer. */
    outr(IO_TC_CC_REG, TC_SWTRG);

    /* now init irmp */
    irmp_init ();
}

/*!
 */
unsigned char IR_Read(u_long tmo)
{
    /* Wait for an event from the interrupt handler. */
    if (NutEventWait(&event_que, tmo) == 0) {
        /* copy the value to a local variable. */
        int returnValue = last_command;
        return returnValue;
    }
    return 0;
}
